/*
 * fancysockcallbackinterfaceaware.h
 *
 *  Created on: 11.08.2009
 *      Author: wglas
 */

#ifndef FANCYSOCKCALLBACKINTERFACEAWARE_H_
#define FANCYSOCKCALLBACKINTERFACEAWARE_H_

#include <fancysock.h>
#include <fancyreflectiontarget.h>

namespace Fancy {

namespace Sock {

/**
 * A common base interface for remote invocation handler factories and client-side
 * bean importers, which are aware of callback interfaces, which are proxied, when passed
 * over on the wire.
 */
class FANCYSOCK_API CallbackInterfaceAware: public virtual Fancy::Object {
protected:
    CallbackInterfaceAware();
    virtual ~CallbackInterfaceAware();
public:
    typedef Fancy::ObjPtr<CallbackInterfaceAware> ptr_t;

    /**
     * Define an interface, for which the server has to issue a callback to the
     * client.
     *
     * For parameters of the given serial ID, the invocation handler generates a
     * proxy, which implements the interface method.
     *
     * @param serialID The serial ID (java class name) of the interface, which will be proxied.
     * @param reflectFunc The pointer to the static @c reflectObject function of the remote interface.
     * @param newProxyFunc The pointer to the static @c newProxyObject function of the remote interface.
     */
    virtual void addCallbackInterface(const char *serialID, Fancy::reflectObject_t reflectFunc, Fancy::newProxyObject_t newProxyFunc) = 0;

    /**
     * Define an interface, for which the server has to issue a callback to the
     * client.
     *
     * For parameters of the type, the invocation handler generates a
     * proxy, which implements the interface method.
     *
     * @param Bean The tyoe of the interface, which will be proxied.
     */
    template <class Bean>
    void addBeanCallbackInterface() {
        this->addCallbackInterface(Bean::getSerialID_s(),Bean::reflectObject,Bean::newProxyObject);
    }

    /**
     * Define an interface, which returns a callback proxy UID to the client instead
     * of the server-side implementation.
     *
     * If you introduce a return callback interface, you have to set
     * a sub system under which to reach an invocation handler for the returned proxies
     * using @c setReturnCallbackSubSystem().
     *
     * @param serialID The serial ID (java class name) of the interface, which will be proxied.
     * @param reflectFunc The pointer to the static @c reflectObject function of the remote interface.
     * @param reflectFunc The pointer to the static @c reflectObject function of the remote interface.
     * @param The name of the function, which will free server-side resources.
     */
    virtual void addReturnCallbackInterface(const char *serialID,
                                            Fancy::reflectObject_t reflectFunc,
                                            Fancy::newProxyObject_t newProxyFunc,
                                            const char *closeFunction) = 0;

    /**
      * Define an interface, which returns a callback proxy UID to the client instead
      * of the server-side implementation.
      *
      * If you introduce a retrun callback interface, you have to register the
      * @c InvocationHandler returned by @c newReturnCallbackHandler() with your
      * @c RemotingConnectionHandlerFactory.
      *
      * @param Bean The bean class, which will be registered.
      * @param The name of the function, which will free server-side resources.
      */
    template <class Bean>
    void addBeanReturnCallbackInterface(const char *closeFunction) {
        this->addReturnCallbackInterface(Bean::getSerialID_s(),Bean::reflectObject,Bean::newProxyObject,closeFunction);
    }
};

}

}

#endif /* FANCYSOCKCALLBACKINTERFACEAWARE_H_ */
