// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockdaemon.h 655 2010-05-19 14:00:09Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie Gmbh.
**
** Author: Wolfgang Glas
**
** A class, that encapsulates a full-fledged UNIX-daemon.
**
*****************************************************************************/

#ifndef FANCYSOCK_DAEMON_H
#define FANCYSOCK_DAEMON_H

#include <fancyobject.h>
#include <fancysockstateful.h>

#ifdef WIN32
extern "C" {
  typedef int fancysock_main_func_t(int argc, char** argv);
  int FancySockServiceMain(int argc, char** argv);
  typedef struct HINSTANCE__ *HINSTANCE;
  int __stdcall WinMain(HINSTANCE, HINSTANCE, char *, int);
}

#define FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,c)                  \
__stdcall WinMain( HINSTANCE instance, HINSTANCE prevInstance, \
		      char *  cmdParam, int cmdShow ) \
{ return Fancy::Sock::impl::ServiceDispatch(cmdParam,c,FancySockServiceMain); } \
int FancySockServiceMain(a,b)
#define FANCYSOCK_DAEMON_MAIN(a,b)  FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,false)
#define FANCYSOCK_DAEMON_MAIN_INTERACTIVE(a,b) FANCYSOCK_DAEMON_MAIN_DISPATCH(a,b,true)
#else
#define FANCYSOCK_DAEMON_MAIN(a,b) main(a,b)
#define FANCYSOCK_DAEMON_MAIN_INTERACTIVE(a,b) main(a,b)
#endif

namespace Fancy {

class Channel;

  namespace Sock {
#ifdef WIN32
  namespace impl {
    FANCYSOCK_API int ServiceDispatch(char *cmdParam, bool interactive,
                                      fancysock_main_func_t *main_func);

  }
#endif

    class EmailClientTransport;

    /**
     * An instance of this class should be kept in a @c main subroutine
     * of portable daemons.
     *
     * If you use this class under Windows, please use the macro
     * @c FANCYSOCK_DAEMON_MAIN for writing you main function and link the
     * application as a window executable (-Wl,--subsystem,windows linker option).
     *
     * @verbatim
     * int FANCYSOCK_DAEMON_MAIN(int argc, char** argv) {
     *
     *   // ... initialize logging, parse cmd line ...
     *
     *   try {
     *      Fancy::Sock::Daemon::ptr_t daemon = Fancy::Sock::Daemon::newInstance("mydaemon");
     *
     *      daemon->instantiate();
     *
     *      // ... initialize daemon subsystems, connect to database ...
     *
     *      daemon->leaveConsole();
     *
     *      // ... daemon main loop ...
     *
     *   } catch (std::exception &e) {
     *      // ... log exception ...
     *   }
     * }
     * @endverbatim
     *
     * A program written this way may be registered by the Windows service manager using
     * @verbatim
     * mydaemon.exe /install mydaemon /pretty "My daemon" /desc "A description" [/user <SvcUser>] [/depends svc1[,svc2[,...]]] [myStartOptions]
     * @endverbatim
     * If @c /user is given, the installer ask you for the user's password before registering the service,
     * @c /depends allows you specify services upon which your daemon depends.
     *
     * In order to unistall a Windows service use
     * @verbatim
     * mydaemon.exe /uninstall mydaemon
     * @endverbatim
     *
     * For later tuning edit the registry, the service information is under this key:
     * @verbatim
     * HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\mydaemon
     * @endverbatim
     *
     * In Windows there are two ways into your FANCYSOCK_DAEMON_MAIN() resp. main():
     * - through the service manager, i.e. called with /service or /instance,
     *   in which case isDaemonEnvironment_s() returns true and your main is supposed to start to service.
     * - through an uninformed or wild user (double click or command line),
     *   in which case isDaemonEnvironment_s() returns false and your main should blame the user and exit.
     *   fprintf(stderr, ...) will only work under mingw shell, better use i.e. MessageBoxA
     *
     */
    class FANCYSOCK_API Daemon : virtual public Stateful
    {
    public:
    	typedef Fancy::ObjPtr<Daemon> ptr_t;

        /**
          Creates a daemon instance and initializes the name of the lockfile
          to @c /var/run/_name_.pid, where _name_ is the name given as the argument.

          Note, that name should be not longer than 26 characters, otherwise
          the filename will be truncated to an overall size of 40 characters.
        */
    	static ptr_t newInstance(const char *name);

    	/**
    	 * This function returns whether the main programm is called under the regime
    	 * of the Windows Service Manager. This is the case, when @c WinMain is called
    	 * with @verbatim /service <servicename> @endverbatim as the first two arguments.
    	 *
    	 * Under UNIX-like operating systems @c true is returned, because daemon operation
    	 * is assumed to be the default operating mode. Programs should parse their command
    	 * line for a special option, which turn off the daemon support under UNIX.
    	 */
    	static bool isDaemonEnvironment_s();

        /**
          Kills a running instance of a daemon, which holds a lock on
          @c /var/run/_name_.pid.

          This subroutine never exits.
          Any error logging is done to stderr.

        */
    	virtual void killInstance() = 0;

        /**
          This function prints a user-readable status report for a running daemon
          instance holding the lock file \c /var/run/_name_.pid to stdout.

          This subroutine never exits.
          Any error logging is done to stderr.
        */
    	virtual void printInstanceStatus() = 0;

        /**
          This function prints a user-readable status report for a running daemon
          instance holding the lock file \c /var/run/_name_.pid to the given channel.

    	  This subroutine exits normally and throw an exception upon errors.
        */
    	virtual void printInstanceStatus(const Fancy::ObjPtr<Fancy::Channel> &channel) = 0;

        /**
          This function mails a user-readable status report for daemons as retrieved
          through @c getStatus(). At the top of the e-mail, additional @c info
          may be attached.
        */
    	virtual void mailStatus(const std::string &info,
    			                const Fancy::ObjPtr<EmailClientTransport> &transport) const = 0;

        /**
          Instantiates a daemon by doing a double-fork and checking for another
          instance holding a lock on the pid-file.

          If the optional argument su_user is given, the effective user id of thw
          forked process is set to the given given user name.

          Returns, if we are the forked child process, which now holds the lock on
          \c /var/run/_name_.pid.

          If an error occurrs, exit() is called and this subroutine never exits.

          Any error logging is done to stderr.

          The next step is calling leaveConsole(), if all initializations which
          are performed after the fork successfully have been finished.

        */
    	virtual void instantiate(const char *su_user=0, unsigned timeout=10) = 0;

    	/**
    	 * Add another stateful object to be added to the daemon's status report.
    	 *
    	 * @param stateful The reporting stateful object to be added.
    	 */
    	virtual void addStateful(const Stateful::ptr_t &stateful) = 0;

    	/**
    	 * Remove a stateful object previously added by @c addStateful().
    	 * Please note, that the current implementation is @c O(n) and thus
    	 * may perform badly for thousands of Stateful's registered.
    	 * Registering many Statefuls might be a bad idea anyways, because
    	 * the status report should be human readable.
    	 */
    	virtual void removeStateful(const Stateful::ptr_t &stateful) = 0;

        /**
          Start @c nprocesses subprocess, which are monitored by a main process.

    	  If @c nprocesses==0, -1 i s returned and this function does nothing.
    	  If this is the main process, this functions returns when the process received
    	  A HUP signal or a request from the Windows Service manager to stop and
    	  when the last child has finished.

          If this is a subprocess (return value > 0) the next action to be performed
          is @c leaveConsole(), which is typically called after setting up the rest of
          the subsystems.

          If this is the main process @c fork() exits with a zero return value after the
          last subprocess has been stopped upon a stop request by the service manager or
          by a UNIX signal. In this case, the main() subroutine should be left immediately.

          @return 0 for the main process and 1,2,... for the subprocesses.
        */
    	virtual int fork(int nprocesses) = 0;

    	/**
    	 * @return The instance number of this process. A value of @c -1 signifies,
    	 *         that this is a single-process daemon. A value of @c 0 means, that
    	 *         this is the main process. Values greater than zero tell which
    	 *         subprocess this is. This function may be called before calling
    	 *         @c fork(int) in order to find out, that you are in a subprocess
    	 *         on operating systems, where @c fork(int) is implemented by calling
    	 *         the executable again with special options.
    	 */
    	virtual int getInstanceCount() const = 0;

        /**
          Detaches the forked daemon process from the console. Up to the call
          to this subroutine, any error logging can be performed using \c stderr.
          The default libxml error handler is redirected from stderr to syslog().

          Additionally, the starter process is informed about the successful start
          of the daemon.

          This call is typically called just before we enter the main loop of the
          daemon.
        */
    	virtual void leaveConsole() = 0;

    	/**
    	 * @return Whether this daemon has already left the console via @cs leaveConsole().
    	 */
    	virtual bool consoleLeft() const = 0;

    protected:
    	Daemon ();

        /**
          Deletes a daemon instance and releases the lockfile and removes it from
          the filesystem.

          If leaveConsole() has not been called yet, the starter process is informed
          about the premature stop of the daemon.
        */
    	virtual ~Daemon ();
    };
  }
}
 
#endif
