// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockdevice.h 938 2011-11-25 22:12:25Z fancylibs $
**
** Copyright (C) 2003 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A base class, which wraps TCP/IP sockets for use with libxml.
** Extensible for using SSL or socks etc. by using many virtual functions.
**
*****************************************************************************/

#ifndef FANCYSOCK_DEVICE_H
#define FANCYSOCK_DEVICE_H

#include <fancypollablechannel.h>
#include <fancysockaddress.h>

namespace Fancy {
  namespace Sock {

  /** A base class for sockets.

    This class defines an socket device, which is represented by
    a POSIX filedecriptor as returned by socket(2).

    It may be reimplemented in order to use many other socket encapsulations
    such as SSL or SOCKS.
  */
  class FANCYSOCK_API Device : virtual public Fancy::PollableChannel
  {
  public:
	  typedef Fancy::ObjPtr<Device> ptr_t;

	  /**
	   * This variant does not allow to inject a soRecvTimeout early enough,
	   * therefore production code need to use DeviceFactory::connect()
	   */
	  static ptr_t connect(const Address::ptr_t &addr);

	  /**
	   * This variant does not allow to inject a soRecvTimeout early enough,
	   * therefore production code need to use DeviceFactory::bind()
	   */
	  static ptr_t bind (const Address::ptr_t &addr);

	  virtual void listen (int backlog) = 0;
	  virtual Device::ptr_t accept() = 0;

	  virtual Address::ptr_t getAddr() const = 0;

	  /**
	   * Set the receive timeout of the socket in milliseconds as per the SO_RECVTIMEO
	   * socket option under UNIX.
	   * If this is set for an accepting Device it is not applied to the accepting socket itself
	   * but forwarded to the accepted Device right after raw accept().
	   */
	  virtual void setSoRecvTimeout(int ms) = 0;

	  /**
	   * Get the receive timeout of the socket in milliseconds as per the SO_RECVTIMEO
	   * socket option under UNIX.
	   */
	  virtual int getSoRecvTimeout() const = 0;

  protected:
	  Device() {}
	  virtual ~Device();
  };

  /**
   * A factory for devices. Server and client pool implementations will use
   * a DeviceFactory in order to switch between unencrypted or SSL communication.
   */
  class FANCYSOCK_API DeviceFactory : virtual public Fancy::Object {

  protected:
	  DeviceFactory() {}
	  virtual ~DeviceFactory();

  public:
	  typedef Fancy::ObjPtr<DeviceFactory> ptr_t;

	  /**
	   * Return a new default instance, which simply calls @c Device::connect() and
	   * @c Device::bind().
	   */
	  static ptr_t newInstance();

	  /**
	   * Connect to the given address and return a client socket.
	   */
	  virtual Device::ptr_t connect(const Address::ptr_t &addr) = 0;

	  /**
	   * Bind to the given address and return a server socket.
	   */
	  virtual Device::ptr_t bind(const Address::ptr_t &addr) = 0;

	  /**
	   * If ms>0, the recv timeout is applied to all new sockets right after raw connect.
	   */
	  virtual void setSoRecvTimeout(int ms) = 0;

	  /**
	   * Returns the value previously set by setSoRecvTimeout(int).
	   */
	  virtual int getSoRecvTimeout() = 0;

  };

  }
}

#endif // ! FANCYSOCK_DEVICE_H
