// This may look like C code, but it is really -*- C++ -*-
/****************************************************************************
** $Id: fancysockhttptransport.h 768 2010-08-17 08:46:00Z fancylibs $
**
** Copyright (C) 2005 EV-i Informationstechnologie GmbH
**
** Author: Wolfgang Glas / EV-i
**
** A minimal HTTP/1.1 implementation for transporting arbitrary documents.
** 
*****************************************************************************/

#ifndef FANCYSOCK_HTTP_TRANSPORT_H
#define FANCYSOCK_HTTP_TRANSPORT_H

#include <fancysocktransport.h>
#include <fancysockexception.h>

namespace Fancy {
  namespace Sock {

    class FANCYSOCK_API HttpException : public Exception
    { 
    public:
      HttpException (fancylog_service *service,
                     int facility,
                     int priority,
                     int reason,
                     const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,6,7)))
#endif
        ;

      HttpException (int facility,
                     int priority,
                     int reason,
                     const char *fmt, ...) throw()
#ifdef __GNUG__
        __attribute__((format(printf,5,6)))
#endif
        ;

      virtual ~HttpException() throw();

      int getReason() const throw();

    private:
      int r;
    };

    class FANCYSOCK_API HttpTransport : virtual public Transport
    {
    protected:
      HttpTransport();

      virtual ~HttpTransport();
    public:

      typedef Fancy::ObjPtr<HttpTransport> ptr_t;

      virtual std::string getRequestMethod() const = 0;
      virtual std::string getRequestUri() const = 0;
      virtual std::string getRequestQuery() const = 0;

      /**
       * The content type of a received document, which is set inside
       * @c openReadDoc(int). This is the value of the @c content-type
       * header field without a possible @c charset=... extension.
       *
       * For writing documents you should pass the @c content-type
       * header as part of the header map to @c openWriteDoc() yourself.
       */
      virtual std::string getContentType() const = 0;

      /**
       * The encoding of the content. This is the value of a
       * @c charset=... extension of the @c content-type header.
       */
      virtual std::string getContentEncoding() const = 0;

      /**
       * The HTTP response reason code sent by the server.
       */
      virtual int getResponseReason() const = 0;

      /**
       *
       */
      virtual int getResponseTTL() const = 0;
    };

    class FANCYSOCK_API HttpClientTransport : virtual public HttpTransport
    {
    protected:
      HttpClientTransport();

      virtual ~HttpClientTransport();
    public:

      typedef Fancy::ObjPtr<HttpClientTransport> ptr_t;

      enum AuthorizationScheme { None, Basic, Digest };

      /**
       * Get a new default clinet implementation.
       *
       * @param d The device to use. It is recommended to set the
       *          receive timeout on the device using @c Device::setSoRecvTimeout(int)
       *          before passing it to this method in order to avoid stalled
       *          client connections with unresponsive servers.
       * @param host The hostname, which is transfered as a @c Host header.
       * @param bufsize The internal buffer size used to transfer data.
       *          The default value should be OK for almost any application.
       */
      static ptr_t newInstance(const Fancy::ObjPtr<Device> &d,
    		                   const std::string &host,
                               int bufsize = 16382);

      /**
       * Set whether we are announcing ourselves as a HTTP/1.0 client.
       * This might be especially interesting, when writing post data
       * to the server, because in HTTP/1.1 mode, the post-data is transfered in
       * chunked encoding, which may not be understood HTTP/1.0 servers.
       *
       * The default value is @c false, which sets the HTTP/1.1 mode.
       */
      virtual void setV10Client(bool b) = 0;
      virtual bool isV10Client() const = 0;

      /**
       * Set whether "Connection: close" is to be used.
       * If "Connection: close" was received, the flag is set, too.
       *
       * The default value is @c false, which sets the HTTP/1.1 mode.
       */
      virtual void setConnectionClose(bool b) = 0;
      virtual bool isConnectionClose() const = 0;


      /**
       * Set the request method before calling @c openWriteDoc().
       *
       * @param method The method, which should be "GET" or "POST".
       */
      virtual void setRequestMethod(const std::string &method) = 0;

      /**
       * Set the request URI before calling @c openWriteDoc().
       *
       * @param uri The URI without any query part.
       */
      virtual void setRequestUri(const std::string &uri) = 0;

      /**
       * Set the request query before calling @c openWriteDoc().
       *
       * @param query The query without any query part.
       */
      virtual void setRequestQuery(const std::string &query) = 0;

      /**
       * Sets the authentication information for HTTP Authorization header.
       *
       * Some schemes may not be implemented or not be supported by the server, so expect Exceptions.
       */
      virtual void setAuthorization(AuthorizationScheme scheme, const std::string &user, const std::string &password) = 0;

    };

    class FANCYSOCK_API HttpServerTransport : virtual public HttpTransport
    {
    protected:
      HttpServerTransport();
    
      virtual ~HttpServerTransport();
    public:

      typedef Fancy::ObjPtr<HttpServerTransport> ptr_t;

      /**
       * Get a new default server implementation.
       *
       * @param d The device to use. It is recommended to set the
       *          receive timeout on the device using @c Device::setSoRecvTimeout(int)
       *          before passing it to this method in order to avoid stalled
       *          server connections with unresponsive clients.
       * @param bufsize The internal buffer size used to transfer data.
       *          The default value should be OK for almost any application.
       */
      static ptr_t newInstance(const Fancy::ObjPtr<Device> &d,
                               int bufsize = 16382);

      /**
       * @return Whether the client is a HTTP/1.0 client and this transport object
       *         has to be purged after writing the response document.
       */
      virtual bool haveV10Client() const = 0;
    
      /**
       * A short cut method for setting the given response reason and writing
       * the given utf-8 encoded string as the response body.
       */
      virtual void writeError(int reason, const std::string &msg) = 0;
      virtual void writeError(const HttpException &e) = 0;

      virtual void setResponseReason(int r) = 0;

      virtual void setResponseTTL(int r) = 0;
    };
  }
}

#endif // ! FANCYSOCK_HTTP_TRANSPORT_H
